import typer
from typing import Optional

from scrapse.leggitalia import leggitalia_app

__version__ = "0.2.2"

app = typer.Typer()
app.add_typer(leggitalia_app, name='leggitalia')


def version_callback(value: bool):
    if value:
        print(f"ScrapSE {__version__}")
        raise typer.Exit()


@app.callback()
def main(
        version: Optional[bool] = typer.Option(
            None, "--version", callback=version_callback
        )):
    """
        Package created for the extraction of judgments.
    """


if __name__ == "__main__":
    app()



from pathlib import Path
import typer
from rich.console import Console

LOCATIONS = ['asti', 'alessandria', 'cuneo', 'ivrea', 'novara', 'torino']

SECTIONS = [
    'Sez. Unite', '(Ad. Plen.)', 'Sez. I', 'Sez. I bis', 'Sez. I ter', 'Sez. I quater', 'Sez. II',
    'Sez. II bis', 'Sez. II ter', 'Sez. II quater', 'Sez. III', 'Sez. III bis', 'Sez. III ter',
    'Sez. III quater', 'Sez. IV', 'Sez. IV bis', 'Sez. IV ter', 'Sez. IV quater', 'Sez. V', 'Sez. V bis',
    'Sez. VI', 'Sez. VI - 1', 'Sez. VI - 2', 'Sez. VI - 3', 'Sez. VI - Lavoro', 'Sez. VI - 5', 'Sez. VII',
    'Sez. VIII', 'Sez. IX', 'Sez. X', 'Sez. XI', 'Sez. XII', 'Sez. XIII', 'Sez. XIV', 'Sez. XV', 'Sez. XVI',
    'Sez. XVII', 'Sez. XVIII', 'Sez. XIX', 'Sez. XX', 'Sez. XXI', 'Sez. XXII', 'Sez. XXIII', 'Sez. XXIV',
    'Sez. XXV', 'Sez. XXVI', 'Sez. XXVII', 'Sez. XXVIII', 'Sez. XXIX', 'Sez. XXX', 'Sez. XXXI', 'Sez. XXXII',
    'Sez. XXXIII', 'Sez. XXXIV', 'Sez. XXXV', 'Sez. XXXVI', 'Sez. XXXVII', 'Sez. XXXVIII', 'Sez. XXXIX',
    'Sez. XL', 'Sez. XLI', 'Sez. XLII', 'Sez. XLIII', 'Sez. XLIV', 'Sez. XLV', 'Sez. XLVI', 'Sez. XLVII',
    'Sez. XLVIII', 'Sez. XLIX', 'Sez. L', 'Sez. LI', 'Sez. LII', 'Sez. LIII', 'Sez. LIV', 'Sez. LV', 'Sez. LVI',
    'Sez. LVII', 'Sez. LVIII', 'Sez. LIX', 'Sez. LX', 'Sez. LXI', 'Sez. LXII', 'Sez. LXIII', 'Sez. LXIV',
    'Sez. LXV', 'Sez. LXVI', 'Sez. LXVII', '(Ad. Gen.)', 'Grande Sez.', 'Sez. App.', 'Sez. I App.',
    'Sez. II App.', 'Sez. III App.', 'Sez. agraria', 'Sez. atti norm.', 'Sez. Autonomie', 'Sez. comm. spec.',
    'Sez. consult.', 'Sez. contr.', 'Sez. contr. enti', 'Sez. Contr. gestione', "Sez. Contr. Legittimita'",
    'Sez. Contr. Stato', 'Sez. disciplinare', 'Sez. enti loc.', 'Sez. fall.', 'Sez. feriale', 'Sez. giurisdiz.',
    'Sez. lavoro', 'Sez. minori', 'Sez. riunite', 'Sez. Riunite cons.', 'Sez. Riunite contr.',
    'Sez. spec. in materia di imprese', 'Sez. spec. propr. industr. ed intell.', 'Sez. stralcio', 'II Stralcio',
    'Sez. uff. elettorale', 'Sez. Unica', 'Sez. I ampliata', 'Sez. II ampliata', 'Sez. III ampliata',
    'Sez. IV ampliata', 'Sez. V ampliata', 'VIII ampliata', 'Sez. VI ampliata', 'Sez. VII ampliata',
    'Grande Cam.', 'Sez. Spec. Immigr., Prot. Inter. e Libera circ. UE',
    'Sez. delle persone, dei minori e della fam.']

JUDICIAL_BODIES = [
    'Corte Costituzionale', 'Cassazione Civile', 'Cassazione Penale', 'Consiglio di Stato',
    'Collegio arbitrale', 'Corte di giustizia tributaria di primo grado',
    'Corte di giustizia tributaria di secondo grado', 'Commiss. Trib. I grado',
    'Commiss. Trib. II grado', 'Commissione Tributaria Centrale', 'Commissione tributaria provinciale',
    'Comm.Tributaria regionale', 'Commiss. usi civici', 'C.G.A. della Regione Siciliana',
    "Corte d'Appello", "Corte d'Appello militare", "Corte d'Assise", "Corte europea diritti dell'uomo",
    "Corte d'Assise d'Appello", 'Corte dei Conti', 'Corte di Giustizia CE',
    'Corte giustizia Unione Europea', "Autorita' Garante per la concorrenza", 'Giudice Istruttore',
    'Giudice di Pace', 'Giudice tutelare', 'T.A.R.', 'T.R.G.A.', 'Tribunale', "Trib. liberta'",
    'Tribunale Militare', 'Tribunale Minorenni', 'Trib. Reg. acque', 'Trib. Sorveglianza',
    'Tribunale Superiore delle Acque Pubbliche', 'Tribunale di Primo Grado CE',
    'Trib. I Grado Unione Europea', 'Uff. indagini preliminari']


class LeggiDiItalia:
    baseurl = 'https://pa.leggiditalia.it/rest'
    headers = {
        'Accept': 'application/json',
        'Content-type': 'application/x-www-form-urlencoded',
    }

    search_params = {
        'core': 'giuryPEL',
        'method': 'solr.search',
        'sort': 'docorder asc:titolo asc:elemorder asc:_is_parent desc',
        'facet_name': 'mask-sentenze',
        'reload': 'false',
        'fields': 'risultato:score:estrcomp',
        'score': 'true',
        'highlight': 'document_text',
        'filters': '{"mode":"AND","items":[{"field":"operasez","mode":"IN","value":["01","05","44","46","47","55",'
                   '"59","60","78","07","20","15","42","66","70","88","K0","K1","K3","K5","DA","DB","DC","DD","0H",'
                   '"2H","9H","90","41","43","H0","06","68","WT","WZ","86","QN","QP","Q6","W3","V3","WH","QE","WO",'
                   '"QK","WQ","VZ","WF","QC","W7","V7","WV","QO","XM","VD","WX","VV","WK","VW","WP","QL","WD","QA",'
                   '"WL","QH","WT","QN","X3","QS","86","Q6","XC","QZ","WM","QI","WS","QJ","WG","QD","XF","VI","WR",'
                   '"QM","X8","VA","WU","QW","X9","VB","WY","QX","WE","XA","VC","QB","XB","QY","X1","QR","WZ","QP",'
                   '"XU","VX","S7","Q7","AP"]}]}',
    }

    export_params = {
        'method': 'print.doc',
        'reload': '1',
        'mode': 'print',
        'FORMATO': 'HTM',
        'PAGINA': 'P',
        'NOMEEXPORT': 'Salva_documenti',
    }

    batch_size = 100

    directory_path = None

    def __init__(self, judicial_bodies=None, sections=None, location=None, extension=None,
                 path=Path.home()) -> None:
        super().__init__()
        self.judicial_bodies = judicial_bodies
        self.sections = sections
        self.location = location
        self.path = str(path)
        self.extension = extension
        self.check_filters()
        self.build_headers()

    def check_filters(self):
        if self.judicial_bodies is None and self.sections is None and self.location is None:
            Console(stderr=True).print('No filter was provided.\nUse --help for more info.')
            raise typer.Exit()

    def build_search_query(self, start='0', rows='0'):
        base_query = """{"mode":"AND","items":[{"mode":"AND","items":[{"field":"operasez","mode":"IN","value":["60","46","47","55","78","59","07"]},{"field":"tipo","mode":"EQUAL","value":"giurisprudenza/sentenze"}]}]}"""
        if self.judicial_bodies is not None:
            base_query = base_query[
                         :-2] + ''',{"field":"giuri_ente","mode":"IN","value":[''' + ','.join(
                f'"{item}"' for item in self.judicial_bodies.split(",")) + ''']}''' + base_query[
                                                                                      -2:]
        if self.sections is not None:
            base_query = base_query[
                         :-2] + ''',{"field":"giuri_sezione","mode":"IN","value":[''' + ','.join(
                f'"{item}"' for item in self.sections.split(",")) + ''']}''' + base_query[
                                                                               -2:]
        if self.location is not None:
            base_query = base_query[
                         :-2] + ''',{"field":"localita_ft","mode":"EQUAL","value":"''' + self.location + '''"}''' + base_query[
                                                                                                                    -2:]
        self.search_params['query'] = base_query
        self.search_params['start'] = str(start)
        self.search_params['rows'] = str(rows)
        return self.search_params

    def build_export_query(self, ids):
        self.export_params['ids'] = ids
        return self.export_params

    def build_directory_path(self):
        filters_to_join = [self.judicial_bodies, self.sections, self.location]
        directory = '_'.join(
            ['&'.join(filter.split(',')).replace(' ', '').lower() for filter in filters_to_join if filter is not None])

        self.directory_path = Path('/'.join((self.path, directory)))
        self.directory_path.mkdir(exist_ok=True, parents=True)

    def build_headers(self):
        path = Path('ldi_cookie.txt')
        if path.is_file():
            self.headers['Cookie'] = path.open(mode='r').read()



import math

import typer
import requests
from bs4 import BeautifulSoup
from pathlib import Path
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TimeRemainingColumn, TimeElapsedColumn
from rich import print
from scrapse import utils
from scrapse.utils import LeggiDiItalia
import multiprocessing
from concurrent.futures import ThreadPoolExecutor

leggitalia_app = typer.Typer()


def search_judgments(ldi):
    with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=False,
    ) as progress:
        progress.add_task(description="Searching for judgments...", total=None)
        request = requests.post(url=ldi.baseurl, headers=ldi.headers,
                                params=ldi.build_search_query())

    judgments_found = request.json()['result']['rows']
    progress.console.print(f'\u2713 Found {judgments_found} sentences')
    judgments_to_download = typer.prompt('How many judgments do you want to download?', default=judgments_found,
                                         type=int)
    return judgments_to_download


def build_pagination(ldi, judgments_to_download):
    pagination = list()
    for index in range(0, math.ceil(judgments_to_download / ldi.batch_size), 1):
        start = index * ldi.batch_size
        rows = ldi.batch_size if (judgments_to_download - (
                (index + 1) * ldi.batch_size)) >= 0 else judgments_to_download - (ldi.batch_size * index)
        pagination.append((start, rows))

    # print(pagination)
    return pagination


def export_sentences(ldi, page, progress, task_id):
    progress[task_id] = {"description": "Id extraction", "progress": 0, "total": 1}
    request = requests.post(url=ldi.baseurl, headers=ldi.headers,
                            params=ldi.build_search_query(start=page[0], rows=page[1]))
    progress[task_id] = {"description": "Id extraction", "progress": 1, "total": 1}

    judgments_id = list()
    for item in request.json()['result']['items']:
        judgments_id.append(item['id'])

    formatted_judgments_id = ['|'.join(judgments_id[i:i + ldi.batch_size]) for i in
                              range(0, len(judgments_id), ldi.batch_size)]
    # export_sentences(ldi, sentences_id)
    progress[task_id] = {"description": "Content extraction", "progress": 0, "total": 1}
    request = requests.post(url=ldi.baseurl, headers=ldi.headers, params=ldi.build_export_query(formatted_judgments_id))
    progress[task_id] = {"description": "Content extraction", "progress": 1, "total": 1}
    # print(request.status_code)
    # print(request.text)
    save_sentences(ldi, judgments_id, request.text, progress, task_id)


def save_sentences(ldi, judgments_id, request_text, progress, task_id):
    soup = BeautifulSoup(request_text, 'html.parser')
    for index, sentence in enumerate(soup.find_all("div", {"class": "doc_body"})):
        progress[task_id] = {"description": "Saving judgments", "progress": index, "total": len(judgments_id)}
        output_file = Path(
            '/'.join((f'{ldi.directory_path}', f'{judgments_id[index]}.{ldi.extension}')))
        output_file.write_text(str(sentence))


@leggitalia_app.command()
def save_cookie(cookie: str):
    file_path = '/'.join([str(Path.cwd().absolute()), 'ldi_cookie.txt'])
    output_file = Path(file_path)
    output_file.parent.mkdir(parents=True, exist_ok=True)
    output_file.write_text(str(cookie))


@leggitalia_app.command()
def scrap(
        judicial_bodies: str = typer.Option(None, "--judicial-bodies", "-j", help="Filter judging bodies"),
        location: str = typer.Option(None, "--location", "-l", help="Filter location of the court of reference"),
        sections: str = typer.Option(None, "--sections", "-s", help="Reference section filter, may change by location"),
        # rows: str = typer.Option('ALL', "--rows", "-r", help="Number of judgments to download"),
        path: Path = typer.Option(Path.home(), "--path", "-p",
                                  help="Path where to save downloaded judgments"),
        extension: str = typer.Option("HTM", "--extension", "-e", help="Desired extension of judgments"),

):
    """
        Download the judgments from the platform https://pa.leggiditalia.it/.

        It is necessary to specify at least one filter among: --adjudicating-bodies, --location and --sections.

        Provide an input list with the following formatting: --sections 'par1, par2, ...'
    """
    ldi = LeggiDiItalia(judicial_bodies=judicial_bodies, location=location, sections=sections,
                        path=path, extension=extension)
    judgments_to_download = search_judgments(ldi)
    pagination = build_pagination(ldi, judgments_to_download)
    ldi.build_directory_path()

    with Progress(
            "[progress.description]{task.description}",
            BarColumn(),
            "[progress.percentage]{task.percentage:>3.0f}%",
            TimeRemainingColumn(),
            TimeElapsedColumn(),
            refresh_per_second=5,
    ) as progress:
        futures = []
        with multiprocessing.Manager() as manager:
            _progress = manager.dict()
            overall_progress_task = progress.add_task("[green]Extraction of all judgments:")
            with ThreadPoolExecutor() as executor:
                for index, page in enumerate(pagination):
                    task_id = progress.add_task(description='')
                    futures.append(executor.submit(export_sentences, ldi, page, _progress, task_id))
                while (n_finished := sum([future.done() for future in futures])) < len(futures):
                    progress.update(overall_progress_task, completed=n_finished, total=len(futures))
                    for task_id, update_data in _progress.items():
                        progress.update(task_id, description=update_data["description"],
                                        completed=update_data["progress"], total=update_data["total"])
                # raise any errors:
                for future in futures:
                    future.result()


@leggitalia_app.command()
def show(j: bool = True, s: bool = True, l: bool = True):
    """
        Shows the possible values to be assigned to the respective filters.
    """
    if j:
        print(f'Judicial bodies {utils.JUDICIAL_BODIES}\n')
    if s:
        print(f'Sections {utils.SECTIONS}\n')
    if l:
        print(f'Locations {utils.LOCATIONS}\n')


@leggitalia_app.callback()
def callback():
    """
       Dedicated command to the site LEGGI D'ITALIA PA
    """


if __name__ == "__main__":
    leggitalia_app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['scrapse']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.11.2,<5.0.0',
 'requests>=2.28.2,<3.0.0',
 'typer[all]>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['scrapse = scrapse.main:app']}

setup_kwargs = {
    'name': 'scrapse',
    'version': '0.2.2',
    'description': 'Package for downloading judgments',
    'long_description': "# ScrapSE\n\n## Package description\n\nScrapSE downloads the judgments in the desired format.  \n\nCurrently supported platforms: LEGGI D'ITALIA PA.\n\n### Install scrapse\n```\npip install scrapse\n```\n\n### How to use\n\n#### Saving cookies - important\n```\nscrapse leggitalia save-cookie 'your_cokies'\n```\nThis command creates a `.txt` file containing `your_cookie`.\n\n#### Show filter values\n```\nscrapse leggitalia show\n```\nThis command shows the possible values to be assigned to sentence search filters.\n\n#### Download the judgments\nMake sure you have **saved** platform-related cookies before downloading the judgments!.\n```\nscrapse leggitalia scrap -l torino -s 'Sez. lavoro, Sez. V'\n```\nThis command creates a folder in `Users/your_username` named `sez.lavoro&sez.v_torino` containing the judgments.\n\n#### For more help\n```\nscrapse --help\n```\n",
    'author': 'zaharia laurentiu jr marius',
    'author_email': 'zaharialorenzo@gmail.com',
    'maintainer': 'zaharia laurentiu jr marius',
    'maintainer_email': 'zaharialorenzo@gmail.com',
    'url': 'https://gitlab.di.unito.it/ngupp/ngupp-scrapse',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.2,<4.0',
}


setup(**setup_kwargs)
